use crate::{change_state, prelude::*};

#[inline]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    name: String,
    _: String,
) -> crate::Result<()> {
    dialogue.exit().await?;

    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    Account::delete(user_id, &name, &db).await?;

    ids.alter_message(
        &bot,
        "The account is successfully deleted",
        deletion_markup(),
        None,
    )
    .await?;

    Ok(())
}

#[inline]
pub async fn delete(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: Pool,
    dialogue: MainDialogue,
    (hash, is_command): (super::NameHash, bool),
) -> crate::Result<()> {
    const TEXT: &str = "Send master password. \
        Once you send the master password the account is unrecoverable";

    let mut ids: MessageIds = q.message.as_ref().unwrap().into();
    let user_id = q.from.id.0;

    let Some(name) = Account::get_name_by_hash(user_id, &hash, &db).await? else {
        bot.send_message(ids.0, "Account wasn't found")
            .reply_markup(deletion_markup())
            .await?;
        bot.answer_callback_query(q.id).await?;
        return Ok(());
    };

    if is_command {
        ids.alter_message(&bot, TEXT, None, None).await?;
    } else {
        let msg = bot.send_message(ids.0, TEXT).await?;
        ids = MessageIds::from(&msg);
    };

    bot.answer_callback_query(q.id).await?;

    change_state!(dialogue, ids, (name), State::GetMasterPass, get_master_pass);

    Ok(())
}
