use crate::prelude::*;
use std::fmt::Write;
use teloxide::types::ParseMode;

/// Handles /`get_accounts` command by sending the list of copyable account names to the user
#[inline]
pub async fn get_accounts(bot: Throttle<Bot>, msg: Message, db: Pool) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let mut account_names = Account::get_names(user_id, &db);

    let mut text = if let Some(name) = account_names.try_next().await? {
        format!("Accounts:\n`{name}`")
    } else {
        bot.send_message(msg.chat.id, "No accounts found")
            .reply_markup(deletion_markup())
            .await?;
        return Ok(());
    };

    while let Some(name) = account_names.try_next().await? {
        write!(text, "\n`{name}`")?;
    }

    bot.send_message(msg.chat.id, text)
        .parse_mode(ParseMode::MarkdownV2)
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}
