use crate::{change_state, prelude::*};
use cryptography::hashing::HashedBytes;
use tokio::task::spawn_blocking;

#[inline]
async fn get_master_pass2(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    hash: HashedBytes<[u8; 64], [u8; 64]>,
    master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;

    if !hash.verify(master_pass.as_bytes()) {
        ids.alter_message(
            &bot,
            "The passwords didn't match. Use the command again",
            deletion_markup(),
            None,
        )
        .await?;

        return Ok(());
    }

    let model = MasterPass {
        user_id,
        password_hash: hash.hash.to_vec(),
        salt: hash.salt.to_vec(),
    };
    model.insert(&db).await?;

    ids.alter_message(&bot, "Success", deletion_markup(), None)
        .await?;

    Ok(())
}

#[inline]
async fn get_master_pass(
    bot: Throttle<Bot>,
    _: Message,
    _: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    master_pass: String,
) -> crate::Result<()> {
    let hash = spawn_blocking(move || HashedBytes::new(master_pass.as_bytes())).await?;

    ids.alter_message(&bot, "Send it again", None, None).await?;

    change_state!(
        dialogue,
        ids,
        (hash),
        State::GetNewMasterPass,
        get_master_pass2
    );

    Ok(())
}

/// Handles /`set_master_pass` command
#[inline]
pub async fn set_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    dialogue: MainDialogue,
    db: Pool,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    if MasterPass::exists(user_id, &db).await? {
        bot.send_message(msg.chat.id, "Master password already exists")
            .reply_markup(deletion_markup())
            .await?;
        return Ok(());
    }
    let previous = bot
        .send_message(msg.chat.id, "Send new master password")
        .await?;

    change_state!(
        dialogue,
        &previous,
        (),
        State::GetNewMasterPass,
        get_master_pass
    );

    Ok(())
}
